unit uRedBlackTree;

interface

uses
  Graphics, ExtCtrls, SysUtils, Math, Classes, Controls, Forms,
  Dialogs, Buttons, Contnrs, TypInfo, uRedBlackTreeNode;

type
  TNodeHandlerProcedure = procedure(node: TRedBlackTreeNode);

  TRedBlackTree = class

  private
    _root: TRedBlackTreeNode;

    _image: TImage;
    C: TCanvas;

    _verticalSpacing: Integer;
    _horizontalSpacing: Real;
    _drawNilLeaves: Boolean;
    _drawBackgroundGradient: Boolean;
    _nodeToHighLight: TRedBlackTreeNode;
    _standardMode: Boolean;

    _fontForNilLeaves: TFont;
    _fontForNodes: TFont;

    _snapshots: array of TMemoryStream;
    _actual_snapshot: Integer;

    FOnChange: TNotifyEvent;

    function insertIntoBST(k: Integer):TRedBlackTreeNode;
    procedure insert1(node: TRedBlackTreeNode);
    procedure insert2(node: TRedBlackTreeNode);
    procedure insert3(node: TRedBlackTreeNode);
    procedure insert4(node: TRedBlackTreeNode);
    procedure insert5(node: TRedBlackTreeNode);

    function deleteFromBST(node: TRedBlackTreeNode):TRedBlackTreeNode;
    procedure deleteOneChild(node: TRedBlackTreeNode);
    procedure delete1(node: TRedBlackTreeNode);
    procedure delete2(node: TRedBlackTreeNode);
    procedure delete3(node: TRedBlackTreeNode);
    procedure delete4(node: TRedBlackTreeNode);
    procedure delete5(node: TRedBlackTreeNode);
    procedure delete6(node: TRedBlackTreeNode);

    procedure rotate_left(root: TRedBlackTreeNode);
    procedure rotate_right(root: TRedBlackTreeNode);

    procedure replace_node(a, b: TRedBlackTreeNode);

    procedure draw(node: TRedBlackTreeNode; width, x, y: Integer); overload;
    procedure drawNilLeave(x, y: Integer);
    procedure drawNode(node: TRedBlackTreenode; x, y: Integer);
    procedure drawChilds(node: TRedBlackTreeNode; width, x, y: Integer);
    procedure drawBackground();

    procedure setVerticalSpacing(space: Integer);
    function getVerticalSpacing():Integer;
    procedure setHorizontalSpacing(space: Real);
    procedure setDrawNilLeaves(v: Boolean);
    procedure setDrawBackgroundGradient(v: Boolean);
    procedure setStandardMode(v: Boolean);

    procedure preorder(node: TRedBlackTreeNode; p: TNodeHandlerProcedure);
    procedure inorder(node: TRedBlackTreeNode; p: TNodeHandlerProcedure);
    procedure postorder(node: TRedBlackTreeNode; p: TNodeHandlerProcedure);
    procedure levelorder(node: TRedBlackTreeNode; p: TNodeHandlerProcedure);

    function heightOfSubtree(node: TRedBlackTreeNode):Integer;
    function blackHeightOfSubtree(node: TRedBlackTreeNode):Integer;
    function widthOfSubtree(node: TRedBlackTreeNode):Integer;
    function depthOfNode(node: TRedBlackTreeNode):Integer;

    function minimalNodeOfSubtree(node: TRedBlackTreeNode):TRedBlackTreeNode;
    function maximalNodeOfSubtree(node: TRedBlackTreeNode):TRedBlackTreeNode;

    function minimumOfSubtree(node: TRedBlackTreeNode):Integer;
    function maximumOfSubtree(node: TRedBlackTreeNode):Integer;

    procedure serializeSubtree(node: TRedBlackTreeNode; stream: TStream);
    function deserializeSubtree(stream: TStream):TRedBlackTreeNode;

    procedure makeNewSnapshot();
    procedure wipeOutSnapshots();

    procedure onFontChange(Sender: TObject);


  public

    constructor Create(image: TImage; font_nodes, font_nil_leaves: TFont);

    procedure setRoot(root: TRedBlackTreeNode);

    destructor Destroy(); override;

    procedure insert(k: Integer);
    procedure delete(k: Integer);
    function search(k: Integer):TRedBlackTreeNode;

    procedure draw(); overload;

    function height():Integer;
    function width():Integer;

    function blackHeight():Integer;

    function minimum():Integer;
    function maximum():Integer;

    procedure serialize(filename: String);
    procedure deserialize(filename: String);

    procedure nextSnapshot();
    procedure previousSnapshot();
    function hasNextSnapshot():Boolean;
    function hasPreviousSnapshot():Boolean;

    procedure preorder(p: TNodeHandlerProcedure);
    procedure inorder(p: TNodeHandlerProcedure);
    procedure postorder(p: TNodeHandlerProcedure);
    procedure levelorder(p: TNodeHandlerProcedure);

    procedure clear();

    procedure change();

    procedure highlight(node: TRedBlackTreeNode);

    property root: TRedBlackTreeNode
      read _root
      write setRoot;

    property verticalSpacing: Integer
      read getVerticalSpacing
      write setVerticalSpacing;

    property horizontalSpacing: Real
      read _horizontalSpacing
      write setHorizontalSpacing;

    property drawNilLeaves: Boolean
      read _drawNilLeaves
      write setDrawNilLeaves;

    property drawBackgroundGradient: Boolean
      read _drawBackgroundGradient
      write setDrawBackgroundGradient;

    property standardMode: Boolean
      read _standardMode
      write setStandardMode;

    property fontForNilLeaves: TFont
      read _fontForNilLeaves
      write _fontForNilLeaves;

    property fontForNodes: TFont
      read _fontForNodes
      write _fontForNodes;

    property OnChange: TNotifyEvent
      read FOnChange
      write FOnChange;

  end;

implementation

function grandparent(n: TRedBlackTreeNode):TRedBlackTreeNode;
begin

  if ((n <> NIL) and (n.parent <> NIL)) then
    begin
      Result := n.parent.parent;
    end
  else
    begin
      Result := NIL;
    end;

end;

function uncle(n: TRedBlackTreeNode):TRedBlackTreeNode;
var
  g: TRedBlackTreeNode;
begin

  g := grandparent(n);

  if (g = NIL) then
    begin
      Result := NIL
    end
  else
    begin
      if n.parent = g.left then
        begin
          Result := g.right
        end
      else
        begin
          Result := g.left;
        end;
    end;

end;

function sibling(n: TRedBlackTreeNode):TRedBlackTreeNode;
begin

  if n = n.parent.left then
    begin
      Result := n.parent.right
    end
  else
    begin
      Result := n.parent.left;
    end;
end;

{_________TRedBlackTree__________}

constructor TRedBlackTree.Create(image: TImage; font_nodes, font_nil_leaves: TFont);
begin
  self._image := image;
  self.C := self._image.Canvas;

  self._root := NIL;

  self._verticalSpacing := 50;
  self._horizontalSpacing := 1/2;
  self._drawNilLeaves := True;
  self._drawBackgroundGradient := False;
  self._standardMode := True;

  self._fontForNodes := font_nodes;
  self._fontForNilLeaves := font_nil_leaves;

  self._fontForNilLeaves.OnChange := @self.onFontChange;
  self._fontForNodes.OnChange := @self.onFontChange;

  self.wipeOutSnapshots();
  self.makeNewSnapshot();

  self.change();
end;

destructor TRedBlackTree.Destroy;
begin
  if (self._root <> nil) then
    begin
      self._root.Destroy();
    end;

  inherited;
end;

procedure TRedBlackTree.setRoot(root: TRedBlackTreeNode);
begin
  if (self._root <> nil) then
    begin
      self._root.Destroy();
    end;

  self._root := root;

  self.highlight(NIL);

  self.change();
end;

procedure TRedBlackTree.rotate_right(root: TRedBlackTreeNode);
var
  pivot: TRedBlackTreeNode;
begin

  pivot := root.left;

  root.left := pivot.right;
  pivot.right := root;

  pivot.parent := root.parent;
  root.parent := pivot;

  if root.left <> nil then
    begin
      root.left.parent := root;
    end;

  if pivot.parent <> NIL then
    begin

      if root = pivot.parent.left then
        begin
          pivot.parent.left := pivot;
        end
      else
        begin
          pivot.parent.right := pivot;
        end;

    end
  else
    begin

      self._root := pivot;

    end;

end;

procedure TRedBlackTree.replace_node(a, b: TRedBlackTreeNode); //brutal lenivost
var
  a_left, a_right, a_parent, b_left, b_right, b_parent: TRedBlackTreeNode;
begin

  //if ()

  a_left := a.left;
  a_right := a.right;
  a_parent := a.parent;

  if (b <> NIL) then
    begin
      b_left := b.left;
      b_right := b.right;
      b_parent := b.parent;
    end;

  a.left := b_left;
  a.right := b_right;
  a.parent := b_parent;

  if (b <> NIL) then
    begin
      b.left := a_left;
      b.right := a_right;
      b.parent := a_parent;
    end;
end;

procedure TRedBlackTree.rotate_left(root: TRedBlackTreeNode);
var
  pivot: TRedBlackTreeNode;
begin

  pivot := root.right;

  root.right := pivot.left;
  pivot.left := root;

  pivot.parent := root.parent;
  root.parent := pivot;

  if root.right <> nil then
    begin
      root.right.parent := root;
    end;

  if pivot.parent <> NIL then
    begin

      if root = pivot.parent.left then
        begin
          pivot.parent.left := pivot;
        end
      else
        begin
          pivot.parent.right := pivot;
        end;

    end
  else
    begin

      self._root := pivot;

    end;
end;

procedure TRedBlackTree.insert(k: Integer);
var
  n: TRedBlackTreeNode;
begin
  n := self.insertIntoBST(k);
  self.highlight(n);

  if n <> NIL then
    begin
      self.insert1(n);

      self.makeNewSnapshot();
    end;

  self.change();

end;

function TRedBlackTree.insertIntoBST(k: Integer):TRedBlackTreeNode;
var
  n: TRedBlackTreeNode;
begin

  Result := NIL;

  if self._root = NIL then
    begin
      self._root := TRedBlackTreeNode.Create(k, Black, nil);
      Result := self._root;
    end
  else
    begin
      n := self._root;

      while n <> NIL do
        begin

          if k = n.key then
            begin
              Result := NIL;
              Exit;
            end;

          if k < n.key then
            if n.left = NIL then
              begin
                n.left := TRedBlackTreeNode.Create(k, Red, n);
                Result := n.left;
                Exit;
              end
            else
              n := n.left
          else
            if n.right = NIL then
              begin
                n.right := TRedBlackTreeNode.Create(k, Red, n);
                Result := n.right;
                Exit;
              end
            else
              n := n.right;

        end;
    end;

end;

procedure TRedBlackTree.insert1(node: TRedBlackTreeNode);
begin
  if node.parent = NIL then
    begin
      node.color := Black;
    end
  else
    begin
      self.insert2(node);
    end;
end;

procedure TRedBlackTree.insert2(node: TRedBlackTreeNode);
begin
  if node.parent.color = Black then
    begin
      Exit;
    end
  else
    begin
      self.insert3(node);
    end;
end;

procedure TRedBlackTree.insert3(node: TRedBlackTreeNode);
var
  u, g: TRedBlackTreeNode;
begin

  u := uncle(node);

  if (u <> NIL) and (u.color = Red) then
    begin
      node.parent.color := Black;
      u.color := Black;
      g := grandparent(node);
      g.color := Red;
      self.insert1(g);
    end
  else
    begin
      self.insert4(node);
    end;

end;

procedure TRedBlackTree.insert4(node: TRedBlackTreeNode);
var
  g: TRedBlackTreeNode;
begin

  g := grandparent(node);

  if (node = node.parent.right) and (node.parent = g.left) then
    begin
      rotate_left(node.parent);
      node := node.left;
    end
  else
    begin
      if (node = node.parent.left) and (node.parent = g.right) then
        begin
          rotate_right(node.parent);
          node := node.right;
        end;
    end;

  self.insert5(node);

end;

procedure TRedBlackTree.insert5(node: TRedBlackTreeNode);
var
  g: TRedBlackTreeNode;
begin

  g := grandparent(node);

  node.parent.color := Black;
  g.color := Red;

  if (node = node.parent.left) and (node.parent = g.left) then
    begin
      rotate_right(g);
    end
  else
    begin
      rotate_left(g);
    end;

end;

procedure TRedBlackTree.delete(k: Integer);
var
  node: TRedBlackTreeNode;
begin
  node := self.search(k);

  if (node = NIL) then
    begin
      Exit;
    end;

  self.deleteOneChild(self.deleteFromBST(node));

  self.change();
end;

function TRedBlackTree.deleteFromBST(node: TRedBlackTreeNode):TRedBlackTreeNode;
begin

  Result := node;

  if ((node.left <> NIL) and (node.right <> NIL)) then
    begin
      Result := self.minimalNodeOfSubtree(node.right);
    end
  else
    begin
      if (node.left <> NIL) then
        begin
          Result := self.maximalNodeOfSubtree(node.left);
        end;
      if (node.right <> NIL) then
        begin
          Result := self.minimalNodeOfSubtree(node.right);
        end;
    end;

  if (Result <> node) then
    begin
      node.key := Result.key;
    end
  else
    begin
      FreeAndNil(node);
    end;
end;

procedure TRedBlackTree.deleteOneChild(node: TRedBlackTreeNode);
var
  child: TRedBlackTreeNode;
begin

  if (node.right = NIL) then
    begin
      child := node.left;
    end
  else
    begin
      child := node.right;
    end;

  self.delete1(node.parent);

  self.replace_node(node, child);

  if (node.color = Black) then
    begin
      if (child.color = Red) then
        begin
          child.color := Black;
        end
      else
        begin
          self.delete1(child);
        end;
    end;

  node.Destroy();

end;

procedure TRedBlackTree.delete1(node: TRedBlackTreeNode);
begin

  if (node.parent <> NIL) then
    begin
      self.delete2(node);
    end;

end;

procedure TRedBlackTree.delete2(node: TRedBlackTreeNode);
var
  s: TRedBlackTreeNode;
begin

  s := sibling(node);

  if (s.color = Red) then
    begin
      node.parent.color := Red;
      s.color := Black;

      if (node = node.parent.left) then
        begin
          self.rotate_left(node.parent);
        end
      else
        begin
          self.rotate_right(node.parent);
        end;

    end;

    self.delete3(node);
end;

procedure TRedBlackTree.delete3(node: TRedBlackTreeNode);
var
  s: TRedBlackTreeNode;
begin

  s := sibling(node);

  if ((node.parent.color = Black) and (s.color = Black) and ((s.left = NIL) or (s.left.color = Black)) and ((s.right = NIL) or (s.right.color = Black))) then
    begin
      s.color := Red;
      self.delete1(node.parent);
    end
  else
    begin
      self.delete4(node);
    end;

end;

procedure TRedBlackTree.delete4(node: TRedBlackTreeNode);
var
  s: TRedBlackTreeNode;
begin

  s := sibling(node);

  if ((node.parent.color = Red) and (s.color = Black) and (s.left.color = Black) and (s.right.color = Black)) then
    begin
      s.color := Red;
      node.parent.color := Black;
    end
  else
    begin
      self.delete5(node);
    end;

end;

procedure TRedBlackTree.delete5(node: TRedBlackTreeNode);
var
  s: TRedBlackTreeNode;
begin

  s := sibling(node);

  if (s.color = Black) then
    begin
      if ((node = node.parent.left) and (s.right.color = Black) and (s.left.color = Red)) then
        begin
    			s.color := Red;
    			s.left.color := Black;
    			self.rotate_right(s);
        end
      else
        begin
          if ((node = node.parent.right) and (s.left.color = Black) and (s.right.color = Red)) then
            begin
        			s.color := Red;
        			s.right.color := Black;
        			rotate_left(s);
            end;
        end;
    end;

  self.delete6(node);

end;

procedure TRedBlackTree.delete6(node: TRedBlackTreeNode);
var
  s: TRedBlackTreeNode;
begin

  s := sibling(node);

  s.color := node.parent.color;
  node.parent.color := Black;

  if (node = node.parent.left) then
    begin
      s.right.color := Black;
      self.rotate_left(node.parent);
    end
  else
    begin
      s.left.color := Black;
      self.rotate_right(node.parent);
    end;

end;

procedure TRedBlackTree.preorder(p: TNodeHandlerProcedure);
begin
  if (self.root <> NIL) then
    self.preorder(self.root, p);
end;

procedure TRedBlackTree.preorder(node: TRedBlackTreeNode; p: TNodeHandlerProcedure);
begin

  p(node);

  if node.left <> NIL then
    begin
      self.preorder(node.left, p);
    end;

  if node.right <> NIL then
    begin
      self.preorder(node.right, p);
    end;

end;

procedure TRedBlackTree.inorder(p: TNodeHandlerProcedure);
begin
  if (self.root <> NIL) then
    self.inorder(self.root, p);
end;

procedure TRedBlackTree.inorder(node: TRedBlackTreeNode; p: TNodeHandlerProcedure);
begin

  if node.left <> NIL then
    begin
      self.inorder(node.left, p);
    end;

  p(node);

  if node.right <> NIL then
    begin
      self.inorder(node.right, p);
    end;

end;

procedure TRedBlackTree.postorder(p: TNodeHandlerProcedure);
begin
  if (self.root <> NIL) then
    self.postorder(self.root, p);
end;

procedure TRedBlackTree.postorder(node: TRedBlackTreeNode; p: TNodeHandlerProcedure);
begin

  if node.left <> NIL then
    begin
      self.postorder(node.left, p);
    end;

  if node.right <> NIL then
    begin
      self.postorder(node.right, p);
    end;

  p(node);

end;

procedure TRedBlackTree.levelorder(p: TNodeHandlerProcedure);
begin
  if (self.root <> NIL) then
    self.levelorder(self.root, p);
end;

procedure TRedBlackTree.levelorder(node: TRedBlackTreeNode; p: TNodeHandlerProcedure);
var
  q : TQueue;
  n: TRedBlackTreeNode;
begin
   q := TQueue.Create;

   q.Push(node);

   while (q.Count > 0) do
     begin

       n := TRedBlackTreeNode(q.Pop);

       if (n.left <> nil) then
         begin
           q.Push(n.left);
         end;

       if (n.right <> nil) then
         begin
           q.Push(n.right);
         end;

       p(n);

     end;
end;

procedure TRedBlackTree.draw(node: TRedBlackTreeNode; width, x, y: Integer);
begin

  if node <> NIL then
    begin
      self.drawChilds(node, width, x, y);
      self.drawNode(node, x, y);
    end
  else
    begin
      self.drawNilLeave(x, y);
    end;

end;

procedure TRedBlackTree.drawNode(node: TRedBlackTreeNode; x, y: Integer);
var
  s: String;
  w, h: Integer;
  r: TRect;
begin

  s := IntToStr(node.key);

  self.C.Brush.Color := node.getRGB();

  self.C.Pen.EndCap := pecRound;
  self.C.Pen.JoinStyle := pjsRound;

  self.C.Font.Assign(self.fontForNodes);

  if (node = self._nodeToHighLight) then
    begin
      self.C.Pen.Color := clYellow;
      self.C.Pen.Width := 3;
    end
  else
    begin
      self.C.Pen.Color := node.getRGB();
      self.C.Pen.Width := 1;
    end;

  h := self.C.TextHeight(s);
  w := self.C.TextWidth(s);

  r := Rect(x - (w div 2) - 4, y - (h div 2) - 2, x + (w div 2) + 4, y + (h div 2) + 2);

  self.C.RoundRect(r, h div 3, h div 3);

  self.C.Font.Color := clWhite;
  self.C.TextOut(x - (w div 2), y - (h div 2), s);
end;

procedure TRedBlackTree.drawNilLeave(x, y: Integer);
var
  s: String;
  w, h: Integer;
  r: TRect;
begin
  if (not self.drawNilLeaves) then
    Exit;

  s := 'nil';

  self.C.Brush.Color := clBlack;
  self.C.Pen.Color := clBlack;

  self.C.Font.Assign(self.fontForNilLeaves);

  h := self.C.TextHeight(s);
  w := self.C.TextWidth(s);

  r := Rect(x - (w div 2) - 4, y - (h div 2) - 2, x + (w div 2) + 4, y + (h div 2) + 2);
  self.C.Rectangle(r);

  self.C.Font.Color := clWhite;
  self.C.TextOut(x - (w div 2), y - (h div 2), s);
end;

procedure TRedBlackTree.drawChilds(node: TRedBlackTreeNode; width, x, y: Integer);
begin

  if (node.left <> NIL) or (self.drawNilLeaves) then
    begin
      self.C.Pen.Color := clBlack;
      self.C.Pen.Style := psSolid;
      self.C.Pen.Width := 1;

      self.C.Line(x, y, round(x - (width * self.horizontalSpacing)), y + self.verticalSpacing);
      self.draw(node.left, round(width * self.horizontalSpacing), round(x - (width * self.horizontalSpacing)), y + self.verticalSpacing);

    end;

  if (node.right <> NIL) or (self.drawNilLeaves) then
    begin
      self.C.Pen.Color := clBlack;
      self.C.Pen.Style := psSolid;
      self.C.Pen.Width := 1;

      self.C.Line(x, y, round(x + (width * self.horizontalSpacing)), y + self.verticalSpacing);
      self.draw(node.right, round(width * self.horizontalSpacing), round(x + (width * self.horizontalSpacing)), y + self.verticalSpacing);

    end;
end;

procedure TRedBlackTree.drawBackground();
var
  i, koef: Integer;
  b: Byte;
  s: String;
  r: TRect;
begin
  self.C.Brush.Color := clBtnFace;
  self.C.FillRect(self._image.ClientRect);

  if (not self.drawBackgroundGradient) then
    Exit;

  self.C.Font.Name := 'default';
  self.C.Font.Height := max(1, min(self._verticalSpacing, 12) - 2);
  self.C.Font.Style := [fsBold];

  self.C.Pen.Width := 0;
  self.C.Pen.Style := psClear;

  self.C.Brush.Style := bsSolid;

  b := 0;

  if self.standardMode then
    begin
      r := Rect(0, 0, self._image.Width, self.verticalSpacing);
      koef := 1;
    end
  else
    begin
      r := Rect(0, self._Image.Height, self._image.Width, self._Image.Height + self.verticalSpacing);
      koef := -1;
    end;

  for i := 0 to self._image.Height div self._verticalSpacing + 1 do
    begin
      b := max((50 + i * 5) mod 255, b);
      self.C.Brush.Color := RGBToColor(b, b, b);

      s := IntToStr(i + 1);

      self.C.Rectangle(r);
      self.C.TextOut(10, r.Top + koef * self._verticalSpacing div 2 - C.TextHeight(s) div 2, s);

      r.Top := r.Top + self.verticalSpacing - koef * 1;
      r.Bottom := r.Bottom + self.verticalSpacing - koef * 1;
    end;

end;

procedure TRedBlackTree.draw();
var
  base: Integer;
begin

  self.drawBackground();

  if (self.standardMode) then
    base := 0
  else
    base := self._image.Height;

  self.draw(self._root, round(self._image.Width * self.horizontalSpacing), round(self._image.Width * 0.5), base + (self.verticalSpacing div 2))
end;

procedure TRedBlackTree.setVerticalSpacing(space: Integer);
begin
  self._verticalSpacing := space;
  self.change();
end;

function TRedBlackTree.getVerticalSpacing():Integer;
begin
  Result := self._verticalSpacing;

  if (not self._standardMode) then
    begin
      Result := Result * -1;
    end;
end;

procedure TRedBlackTree.setHorizontalSpacing(space: Real);
begin
  self._horizontalSpacing := space;
  self.change();
end;

procedure TRedBlackTree.setDrawNilLeaves(v: Boolean);
begin
  self._drawNilLeaves := v;
  self.change();
end;

procedure TRedBlackTree.setDrawBackgroundGradient(v: Boolean);
begin
  self._drawBackgroundGradient := v;
  self.change();
end;

procedure TRedBlackTree.setStandardMode(v: Boolean);
begin
  self._standardMode := v;
  self.change();
end;

function TRedBlackTree.heightOfSubtree(node: TRedBlackTreeNode):Integer;
begin
  if node = NIL then
    begin
      Result := 0;
    end
  else
    begin
      Result := max(self.heightOfSubtree(node.left), self.heightOfSubtree(node.right)) + 1;
    end;
end;

function TRedBlackTree.height():Integer;
begin
  Result := self.heightOfSubtree(self._root);
end;

function TRedBlackTree.widthOfSubtree(node: TRedBlackTreeNode):Integer;
var
  q : TQueue;
  n: TRedBlackTreeNode;
  widths: array of Integer;
  base_depth: Integer;
  i: Integer;
  max_width: Integer;
begin

   SetLength(widths, self.heightOfSubtree(node));
   base_depth := self.depthOfNode(node);

   for i := Low(widths) to High(widths) do
     begin
       widths[i] := 0;
     end;

   q := TQueue.Create;

   if node <> nil then
     q.Push(node);

   while (q.Count > 0) do
     begin
       n := TRedBlackTreeNode(q.Pop);

       inc(widths[self.depthOfNode(n) - base_depth]);

       if (n.left <> nil) then
         q.Push(n.left);

       if (n.right <> nil) then
         q.Push(n.right);
     end;

   max_width := 0;

   for i := Low(widths) to High(widths) do
     begin
       max_width := max(widths[i], max_width);
     end;

   Result := max_width;
end;

function TRedBlackTree.width():Integer;
begin
  Result := Self.widthOfSubtree(self._root);
end;

function TRedBlackTree.blackHeightOfSubtree(node: TRedBlackTreeNode):Integer;
begin
  if (node = nil) then
    begin
      Result := 0;
      Exit;
    end;

  if (node.color = Black) then
    begin
      Result := 1 + self.blackHeightOfSubtree(node.left);
    end
  else
    begin
      Result := self.blackHeightOfSubtree(node.left);
    end;
end;

function TRedBlackTree.blackHeight():Integer;
begin
  Result := self.blackHeightOfSubtree(self._root);
end;

function TRedBlackTree.depthOfNode(node: TRedBlackTreeNode):Integer;
var
  depth: Integer;
  n: TRedBlackTreeNode;
begin

  Result := -1;

  if (node = nil) then
    begin
      Result := -1;
      Exit;
    end;

  depth := 0;
  n := self._root;

  while n <> NIL do
    begin

      inc(depth);

      if (node = n) then
        begin
          Result := depth;
          Exit;
        end;

      if node.key < n.key then
        begin
          n := n.left
        end
      else
        begin
          n := n.right;
        end;

    end;

end;

function TRedBlackTree.minimalNodeOfSubtree(node: TRedBlackTreeNode):TRedBlackTreeNode;
begin

  Result := node;

  while ((Result <> NIL) and (Result.left <> NIL)) do
    begin
      Result := Result.left;
    end;

end;

function TRedBlackTree.maximalNodeOfSubtree(node: TRedBlackTreeNode):TRedBlackTreeNode;
begin

  Result := node;

  while (Result.right <> NIL) do
    begin
      Result := Result.right;
    end;
end;

function TRedBlackTree.minimumOfSubtree(node: TRedBlackTreeNode):Integer;
begin
  Result := -1;

  if (node <> NIL) then
    begin
      Result := self.minimalNodeOfSubtree(node).key;
    end;
end;

function TRedBlackTree.maximumOfSubtree(node: TRedBlackTreeNode):Integer;
begin

  Result := -1;

  if (node <> NIL) then
    begin
      Result := self.maximalNodeOfSubtree(node).key;
    end;

end;

function TRedBlackTree.minimum():Integer;
begin
  Result := self.minimumOfSubtree(self._root);
end;

function TRedBlackTree.maximum():Integer;
begin
  Result := self.maximumOfSubtree(self._root);
end;

procedure TRedBlackTree.serialize(filename: String);
var
  f: TFileStream;
begin

  f := TFileStream.Create(filename, fmCreate);

  self.serializeSubtree(self._root, f);

  f.Destroy;

end;

procedure TRedBlackTree.serializeSubtree(node: TRedBlackTreeNode; stream: TStream);
var
  is_nil: Boolean;
begin
  is_nil := (node = nil);

  stream.Write(is_nil, SizeOf(Boolean));

  if (node <> nil) then
    begin
      stream.Write(node.key, SizeOf(node.key));
      stream.Write(node.color, SizeOf(node.color));

      serializeSubtree(node.left, stream);
      serializeSubtree(node.right, stream);

    end;

end;

function TRedBlackTree.deserializeSubtree(stream: TStream):TRedBlackTreeNode;
var
  is_nil: Boolean;

  key: Integer;
  color: TRedBlackTreeNodeColor;
begin

  is_nil := False;
  key := -1;
  color := Black;

  if (stream.Position < stream.Size) then
    begin
      stream.ReadBuffer(is_nil, SizeOf(is_nil));

      if (is_nil) then
        begin
          Result := nil;
        end
      else
        begin
          stream.ReadBuffer(key, SizeOf(key));
          stream.ReadBuffer(color, SizeOf(color));

          Result := TRedBlackTreeNode.Create(key, color, nil);

          Result.left := self.deserializeSubtree(stream);
          Result.right := self.deserializeSubtree(stream);

          if (Result.left <> nil) then
            Result.left.parent := Result;

          if (Result.right <> nil) then
            Result.right.parent := Result;

        end;
    end;

end;

procedure TRedBlackTree.deserialize(filename: String);
var
  f: TFileStream;
begin
  f := TFileStream.Create(filename, fmOpenRead);

  self.wipeOutSnapshots();
  self.root := deserializeSubtree(f);
  self.makeNewSnapshot();

  f.Destroy;
end;

procedure TRedBlackTree.change();
begin

  if Assigned(FOnChange) then
    begin
      FOnChange(Self);
    end;

end;

procedure TRedBlackTree.onFontChange(Sender: TObject);
begin
  self.change();
end;

procedure TRedBlackTree.makeNewSnapshot();
var
  i: Integer;
begin

  for i := self._actual_snapshot + 1 to High(self._snapshots) do
    begin
      FreeAndNil(self._snapshots[i]);
    end;

  SetLength(self._snapshots, self._actual_snapshot + 2);
  inc(self._actual_snapshot);
  self._snapshots[self._actual_snapshot] := TMemoryStream.Create;
  self.serializeSubtree(self._root, self._snapshots[self._actual_snapshot]);

end;

procedure TRedBlackTree.nextSnapshot();
begin

  if self.hasNextSnapshot() then
    begin
      inc(self._actual_snapshot);
      self._snapshots[self._actual_snapshot].Position := 0;
      self.root := self.deserializeSubtree(self._snapshots[self._actual_snapshot]);
    end;

end;

procedure TRedBlackTree.previousSnapshot();
begin

  if self.hasPreviousSnapshot() then
    begin
      dec(self._actual_snapshot);
      self._snapshots[self._actual_snapshot].Position := 0;
      self.root := self.deserializeSubtree(self._snapshots[self._actual_snapshot]);
    end;

end;

function TRedBlackTree.hasNextSnapshot():Boolean;
begin
  Result := (self._actual_snapshot < High(self._snapshots));
end;

function TRedBlackTree.hasPreviousSnapshot():Boolean;
begin
  Result := (self._actual_snapshot > Low(self._snapshots));
end;

procedure TRedBlackTree.wipeOutSnapshots();
var
  i: Integer;
begin

  for i := Low(self._snapshots) to High(self._snapshots) do
    begin
      FreeAndNil(self._snapshots[i]);
    end;

  SetLength(self._snapshots, 0);
  self._actual_snapshot := -1;

end;

procedure TRedBlackTree.clear();
begin
  self.wipeOutSnapshots();
  self.root := nil;
  self.makeNewSnapshot();
end;

function TRedBlackTree.search(k: Integer):TRedBlackTreeNode;
var
  n: TRedBlackTreeNode;
begin

  n := self.root;
  Result := NIL;

  while (n <> NIL) do
    begin

      if (k = n.key) then
        begin
          Result := n;
          Exit;
        end;

      if (k < n.key) then
        begin
          n := n.left;
        end
      else
        begin
          n := n.right;
        end;

    end;
end;

procedure TRedBlackTree.highlight(node: TRedBlackTreeNode);
begin
  self._nodeToHighLight := node;
  self.change();
end;

end.
